CREATE OR REPLACE FUNCTION creation_tu_p(character varying, integer, integer,integer[])
RETURNS void AS
$$
/* 
Cette procdure gnre  partir de la table dXX_YYYY_pnb10_parcelle une table "tache urbaine" (tb_tu_YYYY_dXX) dcompose par priodes 
contact : DREAL Nouvelle-Aquitaine /MiCAT/PIG - M BUFFET 
- version initiale : 11/2012 
- maj 08/2014 (ajout du ndep dans nom table d'export + prise en compte nouvelle variable jannatminh + prise en compte des parcelles composantes des copro multiparcellaires)
- maj 03/12/15 (modif variable nloclog par nloclog)
- maj 10/2016 (utilisation dernire priode YYYY tableau pour choix du millsime  utiliser) 
exemple de commande :  SELECT creation_tu_p('87',40,-15,array[1962,1975,1990,1999,2006,2015]);
PostgresSQL 9.3 - Postgis 2
*/

DECLARE
--dclaration variables  --
dep ALIAS FOR $1;
d1 ALIAS FOR $2;
d2 ALIAS FOR $3;
tb_periode ALIAS FOR $4;
tb varchar;
tb_export varchar;
col varchar;
nb_p integer;
periode varchar;
periode_av varchar;
millesime varchar;
i int2;


BEGIN
millesime:=tb_periode[array_upper(tb_periode,1)];
tb:='ff_d'||dep||'_'||millesime||'.d'||dep||'_'||millesime||'_pnb10_parcelle';
col:=quote_ident('geomloc');

--cration d'une table temporaire des tampons autour des localisants parcellaires--
EXECUTE 'CREATE TEMP table temp_tampon_tu AS
(
	(
	SELECT idpar,idcom,jannatminh ,st_buffer('||tb||'.'||col||','||d1||') as the_geom
	FROM '||tb||' 
	WHERE  '||tb||'.jannatminh>0 AND '||tb||'.nloclog>0 and '||tb||'.'||col||' IS NOT NULL
	)
	UNION
	(
	SELECT tb1.idpar,tb1.idcom,tb2.jannatminh,st_buffer(tb1.'||col||','||d1||') as the_geom
	FROM 
	'||tb||' tb1
	LEFT JOIN '||tb||' tb2
	ON tb1.idparref=tb2.idpar
	WHERE tb1.pdlmp =''C''
	AND tb2.jannatminh>0
	AND tb2.nloclog>0
	)
)';

--cration d'une table de taches urbaines priodiques--
EXECUTE 'CREATE TABLE tp_test_tu(id serial, jannatminh varchar(4), CONSTRAINT "tp_test_tu_pkey" PRIMARY KEY("id"))';
EXECUTE 'SELECT AddGeometryColumn (''tp_test_tu'',''the_geom'',2154,''GEOMETRY'', 2 )';
EXECUTE 'CREATE INDEX geom_tp_tu ON tp_test_tu USING gist (the_geom)';

nb_p:=array_length(tb_periode, 1);
FOR i IN 1..nb_p LOOP
	periode:=tb_periode[i];
	IF i=1 THEN 
	EXECUTE 'INSERT INTO tp_test_tu(jannatminh,the_geom) VALUES ('||periode||',(SELECT st_buffer((SELECT st_union(the_geom) FROM temp_tampon_tu WHERE jannatminh <='||periode||'),'||d2||')))';
	ELSIF i=nb_p THEN
	periode_av:=tb_periode[i-1];
	EXECUTE 'INSERT INTO tp_test_tu(jannatminh,the_geom) VALUES ('||periode||',(SELECT st_buffer((SELECT st_union(the_geom)  FROM temp_tampon_tu WHERE jannatminh >'||periode_av||'),'||d2||')))';
	ELSE 
	periode_av:=tb_periode[i-1];
	EXECUTE 'INSERT INTO tp_test_tu(jannatminh,the_geom) VALUES ('||periode||',(SELECT st_buffer((SELECT st_union(the_geom)  FROM temp_tampon_tu WHERE jannatminh >'||periode_av||' and jannatminh <='||periode||'),'||d2||')))';
	END IF;
END LOOP;

tb_export:= 'tb_tu_'||millesime||'_d'||dep;
-- supression table finale si existante et cration  table --
    IF EXISTS(  
        SELECT *  
        FROM pg_class as tbl  
        INNER JOIN pg_namespace as schm on tbl.relnamespace = schm.oid  
        WHERE schm.nspname = 'public'
        AND tbl.relname = tb_export
    ) THEN  
    	EXECUTE 'DELETE FROM geometry_columns  WHERE f_table_name='||quote_literal(tb_export)||' and f_geometry_column=''the_geom''';
	EXECUTE 'DROP TABLE public.'||tb_export||' CASCADE';
   END IF;  	

        EXECUTE 'CREATE TABLE public.'||tb_export||' AS  TABLE tp_test_tu';
	EXECUTE 'INSERT INTO geometry_columns(f_table_catalog,f_table_schema,f_table_name,f_geometry_column,coord_dimension,srid,type) VALUES('''',''public'','''||tb_export||''',''the_geom'',2,2154,''GEOMETRY'')';
	EXECUTE 'CREATE INDEX '||tb_export||'_'||col||'_gist ON '||tb_export||' USING gist (the_geom)';
	EXECUTE 'ALTER TABLE '||tb_export||' ADD COLUMN surface float';
 
--on met  jour les gomtries de la table finale (partie la plus lente certainement optimisable...) --
FOR i IN 1..nb_p LOOP
	IF i>1 THEN 
	periode:=tb_periode[i];
	EXECUTE 'UPDATE '||tb_export||' SET the_geom = st_difference(the_geom,(SELECT st_union(the_geom) FROM tp_test_tu WHERE jannatminh<'''||periode||''')) where jannatminh = '''||periode||'''';
	END IF;
END LOOP;

--on maj la colonne surface--
EXECUTE'UPDATE '||tb_export||' SET surface = ROUND(st_area2d(the_geom))';

--on met  jour la table geometry_columns  et on supprime les tables temporaires
DELETE FROM geometry_columns  WHERE f_table_name='tp_test_tu' and f_geometry_column='the_geom';
EXECUTE 'DROP TABLE temp_tampon_tu CASCADE';
EXECUTE 'DROP TABLE tp_test_tu CASCADE';
--
END;
$$LANGUAGE plpgsql;